% Figure 1. Co-contraction during maintenance of an upright forearm posture.

clear
clc
close all

%-------------------------------------------------------------------------%
%------------- Problem definition
%-------------------------------------------------------------------------%

ns=2; % dim of state
nu=2; % dim of control
auxdata.ns=ns;
auxdata.nu=nu;

% Parameters for the optimal control problem
auxdata.coef=1e4; wpos=1; wvel=0.1; % variance weight
auxdata.Q=auxdata.coef*[wpos 0 ; 0 wvel];
auxdata.rescale=1e-2 ; % to rescale the cost globally (only for numerical issues)
auxdata.R=diag(ones(1,nu)) ; % effort weight
auxdata.Qf=auxdata.coef*[wpos 0 ; 0 wvel];

% Initial mean and covariance (augmented state = [m,c])
m0=[0 0]'; 
c0=[0 0 0 0]';
auxdata.m0=m0;
auxdata.c0=c0;

% task duration
T=5; % in seconds
tfguess=T;

m5pound=1*2.26796185; % with/out additional mass
g=9.81;
l=0.35;
lc=0.21;
m=1.44;
J=0.0588+m5pound*l^2;
b=1;
G=1; % gain to convert u1-u2 to real net torque
K=G; % gain to convert u1+u2 to joint stiffness
A=[0 1 ; m*g*lc/J+m5pound*g*l/J -b/J];
B=[0 0 ; G/J -G/J];
Ni=[0 0 ; -K/J -0*sqrt(K)/J]; 

auxdata.A=A;
auxdata.B=B;
auxdata.Ni=Ni;
auxdata.G=G;
auxdata.K=K;

auxdata.Atilda=kron(eye(ns),A)+kron(A,eye(ns));
auxdata.Nitilda=kron(eye(nu),Ni)+kron(Ni,eye(nu));

% noise properties
% constant noise magnitude
sigma=0.1; % std of noise on acting in control space
Sigma=[0 ; sigma];
auxdata.Sigma=Sigma;
auxdata.Sigmatilda=kron(Sigma,Sigma);
% signal dependent noise magnitude (if any)
d1=0;
d2=d1;
D1=[0 0 ; d1 0];
D2=[0 0 ; 0 d2];

auxdata.D1=D1;
auxdata.D2=D2;

auxdata.D1tilda=kron(D1,D1);
auxdata.D2tilda=kron(D2,D2);

%-------------------------------------------------------------------------%
%-----------  Find optimal solution (SOOC resolution for a bilinear system)
%-------------------------------------------------------------------------%

run GenMain_pendulum

x=output.result.interpsolution.phase(1).state;
u=output.result.interpsolution.phase(1).control;
time=output.result.interpsolution.phase(1).time;
tf=time(end);

% optimal mean trajectory
q = x(:,1);
qdot = x(:,2);

% optimal open-loop control
u1=u(:,1);
u2=u(:,2);

cocont=K*(u1+u2); % Stiffness originating from cocontraction
taunet=G*(u1-u2); % Net joint torque

%-------------------------------------------------------------------------%
%------------- Simulate trajectories with noise
%-------------------------------------------------------------------------%

N=numel(time);
dt=tf/N;
nrep=500; % number of sample trajectories

for k=1:nrep
    xrun(:,1)=m0;
    for i=1:N-1
      u_=[u1(i);u2(i)];
      xrun(:,i+1)=(eye(ns)+A*dt)*xrun(:,i)+B*u_*dt+ ...
                  (Ni*u1(i)*xrun(:,i)*dt+Ni*u2(i)*xrun(:,i)*dt)+...
                  Sigma*randn()*sqrt(dt)+...
                  D1*u_*randn()*sqrt(dt)+ D2*u_*randn()*sqrt(dt);  
    end
    theta(:,k)=xrun(1,:)';
    thetadot(:,k)=xrun(2,:)';   
end
time2=time;

for ii=1:size(theta,1) 
   thetastd(ii)=std(theta(ii,:));
   thetamean(ii)=mean(theta(ii,:));
   thetadotstd(ii)=std(thetadot(ii,:));
   thetadotmean(ii)=mean(thetadot(ii,:));
end


%-------------------------------------------------------------------------%
%------------- Plot figures
%-------------------------------------------------------------------------%

figure('color','white');
width=12;
height=14;
set(gcf, 'Units', 'centimeters', 'OuterPosition', [10, 10, width, height]);

subplot(221)
hold on;
for ii=1:3
    Xp=[time2' NaN];
    Yp=[theta(:,ii)'*180/pi NaN];
    p = patch(Xp,Yp,'k');
    set(p,'edgecolor','k','linewidth',1,'edgealpha',0.2)
end
plotshaded(time2',[thetamean+thetastd ; thetamean ;thetamean-thetastd]*180/pi,'k'); 
plot(time2',thetamean*180/pi,'linewidth',2,'color','k');
xlim([0 tf])
ylim([-3.5 3.5])
xlabel('Time (s)')
ylabel('Position (deg)');
set(gca,'fontsize', 10);

subplot(222)
hold on;
plot(time2',cocont,'linewidth',2,'color','k');
plot(time2',(m*g*lc+m5pound*g*l)*ones(size(time2')),':k','linewidth',2)
xlim([0 tf])
ylim([0 15])
legend('K(u1+u2)','mgl')
xlabel('Time (s)')
ylabel('Stiffness (Nm/rad)');
set(gca,'fontsize', 10);

subplot(223)
hold on;
for ii=1:3
    Xp=[time2' NaN];
    Yp=[thetadot(:,ii)'*180/pi NaN];
    p = patch(Xp,Yp,'k');
    set(p,'edgecolor','k','linewidth',1,'edgealpha',0.1)
end
plotshaded(time2',[thetadotmean+thetadotstd ; thetadotmean ;thetadotmean-thetadotstd]*180/pi,'k'); hold on;
plot(time2',thetadotmean*180/pi,'linewidth',2,'color','k');
xlim([0 tf])
ylim([-10 10])
xlabel('Time (s)')
ylabel('Velocity (deg/s)');
set(gca,'fontsize', 10);

subplot(224)
hold on;
plot(time2',u1,'linewidth',2,'color','k');
plot(time2',-u2,'linewidth',2,'color',[0.5 0.5 0.5]);
plot(time2',(m*g*lc)*ones(size(time2')),':r','linewidth',2)
xlim([0 tf])
ylim([-10 10])
grid on;
legend('Flexor u1','Extensor u2')
xlabel('Time (s)')
ylabel('Muscle torques (Nm)');
set(gca,'fontsize', 10);
