% Figure 2. Simulations of a pointing movement with the forearm (elbow extension of 1 rad).

clear
clc
close

%-------------------------------------------------------------------------%
%------------- Problem definition
%-------------------------------------------------------------------------%

ns=4; % dim of state: [q, qdot, dtheta, dthetadot]
ns2=ns*(ns+1)/2; % dim of covariance state
nu=2; % dim of control
nw=2; % dim of noise
auxdata.ns=ns;
auxdata.ns2=ns2;
auxdata.nu=nu;

% Parameters for the optimal control problem
auxdata.rescale=1 ; % to rescale the cost globally (only for numerical issues)
auxdata.coef=1;
weightVariance=1e4; wpos=weightVariance; wvel=.01*weightVariance; % variance weight
auxdata.Q=auxdata.coef*diag([0 0 wpos wvel]); 
auxdata.R=diag([1 1]); % effort weight
auxdata.Qf=auxdata.coef*diag([0 0 wpos wvel]);
auxdata.Qnoise=diag(ones(1,nw)); % noise covariance 

% Limb's properties
l=0.21;
ma=1.44;
J=0.072;
b=0;
G=1; % gain for net torque
Ks=1; % gain for stiffness
Kv=sqrt(J*Ks); % gain for damping, we get a damping ratio of 1/2 here
g=0; % gravity acceleration

auxdata.G=G;
auxdata.Ks=Ks;
auxdata.Kv=Kv;
auxdata.J=J;
auxdata.g=g;
auxdata.b=b;
auxdata.ma=ma;
auxdata.l=l;

% initial mean and covariance (augmented state = [m,c])
q0=0; % desired initial position
qdot0=0; % desired initial vel
dtheta0=0; % deviation between real and desired position at t0
dthetadot0=0; % deviation between real and desired vel at t0

m0=[q0 qdot0 dtheta0 dthetadot0]';

auxdata.m0=m0;

mf=[1 0 0 0]'; % amplitude of 1 rad
auxdata.mf=mf;

% Movement duration
T=0.75; % seconds 
tfguess=T;

% noise properties
% magnitude of constant noise
sigma1=0;
auxdata.sigma1=sigma1;
% magnitude of signal dependent noise
d1=0.2;
auxdata.d1=d1;

%-------------------------------------------------------------------------%
%------------- Find optimal solution (SOOC resolution)
%-------------------------------------------------------------------------%

run GenMain_forearmflex

x=output.result.interpsolution.phase(1).state;
u=output.result.interpsolution.phase(1).control;
time=output.result.interpsolution.phase(1).time;
tf=time(end);

% Found initial covariance 
c0=x(1,ns+1:end); Pupper0 = triu(ones(ns)); Pupper0(Pupper0==1) = c0;
P0=(Pupper0+Pupper0')-bsxfun(@times,eye(size(Pupper0)),diag(Pupper0));

% optimal trajectory
thetaopt = x(:,1);
thetadotopt = x(:,2);
dthetaopt = x(:,3);
dthetadotopt = x(:,4); 

% optimal open-loop control
u1opt=u(:,1);
u2opt=u(:,2);

% interpolate to create ui(t) functions
u1optpp = interpolate(time,u1opt,'linear','pp'); u1t = @(t) ppval(u1optpp,t);
u2optpp = interpolate(time,u2opt,'linear','pp'); u2t = @(t) ppval(u2optpp,t);

%-------------------------------------------------------------------------%
%------------- Simulate trajectories with noise
%-------------------------------------------------------------------------%

dt=0.001;
N=fix(tf/dt);
nrep=100;
for k=1:nrep
    
    xrun(:,1)=x(1,1:ns)'+sqrt(diag(P0))*randn(); % initial state found   
    for i=1:N-1
      
      ti=(i-1)*dt;
      q_=xrun(1,i); qdot_=xrun(2,i); dtheta_=xrun(3,i); dthetadot_=xrun(4,i);      
      u1=u1t(ti); u2=u2t(ti);
      Gnoise=[0 0 ; 0 0; 0 0 ; sigma1 d1*u1 ]/J; % noise at dthetaddot level
          
      fdyn= [ qdot_ ; ...
              (G*u1-b*qdot_+ma*g*l*sin(q_))/J ; ...
              dthetadot_ ; ...
              (-Ks*u2*dtheta_-Kv*sqrt(u2)*dthetadot_-b*dthetadot_+ma*g*l*(sin(dtheta_+q_)-sin(q_)))/J  ];
      
      xrun(:,i+1)=xrun(:,i) + fdyn*dt + Gnoise*sqrtm(auxdata.Qnoise)*randn(nw,1)*sqrt(dt); 
      
    end
  
    q_all(:,k)=xrun(1,:)';
    qdot_all(:,k)=xrun(2,:)'; 
    dtheta_all(:,k)=xrun(3,:)';
    dthetadot_all(:,k)=xrun(4,:)';  
end

time2=linspace(0,tf,N)';

for ii=1:size(q_all,1) 
   qstd(ii)=std(q_all(ii,:));
   qmean(ii)=mean(q_all(ii,:));
   qdotstd(ii)=std(qdot_all(ii,:));
   qdotmean(ii)=mean(qdot_all(ii,:));
   dthetastd(ii)=std(dtheta_all(ii,:));
   dthetamean(ii)=mean(dtheta_all(ii,:));
   dthetadotstd(ii)=std(dthetadot_all(ii,:));
   dthetadotmean(ii)=mean(dthetadot_all(ii,:));
end

thetamean=qmean+dthetamean;
thetastd=qstd+dthetastd;

thetadotmean=qdotmean+dthetadotmean;
thetadotstd=qdotstd+dthetadotstd;
StiffnessProfiles(:,1)=Ks*u2t(time);

MeanStiff=mean(Ks*u2t(time)); % mean stiffness
MeanNetTorque=mean(abs(u1t(time))); % mean net torque
MeanPv=max(thetadotopt); % mean peak vel (rad/s)

%-------------------------------------------------------------------------%
%------------- Plot figures
%-------------------------------------------------------------------------%

figure('color','white');
width=20;
height=14;
set(gcf, 'Units', 'centimeters', 'OuterPosition', [10, 10, width, height]);

subplot(231)
hold on;
plotshaded(time2',[thetamean+thetastd ; thetamean ;thetamean-thetastd]*180/pi,'k');
plot(time2',thetamean*180/pi,'linewidth',2,'color','k');
xlim([0 time2(end)])
xlabel('Time (s)')
ylabel('Position (deg)');
ylim([-5 70])
set(gca,'fontsize', 10);

subplot(232)
hold on;
plotshaded(time2',[thetadotmean+thetadotstd ; thetadotmean ;thetadotmean-thetadotstd]*180/pi,'k');
plot(time2',thetadotmean*180/pi,'linewidth',2,'color','k');
xlabel('Time (s)')
ylabel('Velocity (deg/s)');
xlim([0 time2(end)])
ylim([-1 200])
set(gca,'fontsize', 10);

subplot(233)
hold on;
plot(time2',u1t(time2'),'linewidth',2,'color','k');
plot(time2',zeros(size(time2'))+mean(abs(u1t(time2'))),'k:');
xlabel('Time (s)')
ylabel('Net torque (Nm)');
xlim([0 time2(end)])
ylim([-2 2])
set(gca,'fontsize', 10);

subplot(234)
plot(time2',ones(size(time2'))*J,'linewidth',2,'color','k');
xlabel('Time (s)')
ylabel('Inertia [Nm/(rad/s?]');
xlim([0 time2(end)])
ylim([0 0.1])
set(gca,'fontsize', 10);

subplot(235)
hold on;
plot(time2',Ks*u2t(time2'),'linewidth',2,'color','k');
plot(time2',zeros(size(time2'))+mean(Ks*u2t(time2')),'k:');
xlabel('Time (s)')
ylabel('Stiffness [Nm/rad]');
xlim([0 time2(end)])
ylim([-.2 15])
set(gca,'fontsize', 10);

subplot(236)
plot(time2',Kv*sqrt(u2t(time2')),'linewidth',2,'color','k');
hold on;
plot(time2',zeros(size(time2'))+mean(Kv*sqrt(u2t(time2'))),'k:');
xlabel('Time (s)')
ylabel('Damping [Nm/(rad/s)]');
xlim([0 time2(end)])
ylim([-0.01 1.0])
set(gca,'fontsize', 10);

