% Figure 3. Simulations of pointing movements following the experiment of [64].

clear 
clc
close all

%-------------------------------------------------------------------------%
%------------- Problem definition
%-------------------------------------------------------------------------%

ns=2; % dim of state (mean part)
ns2=ns*(ns+1)/2; % dim of state (covariance part)
nu=2; % dim of control
nw=2; % dim of noise

auxdata.ns=ns;
auxdata.ns2=ns2;
auxdata.nu=nu;

% Parameters for the optimal control problem: 
auxdata.coef=250; wpos=1; wvel=.1; % variance weight
auxdata.Q=zeros(ns); 
auxdata.R=diag([1 1]); % effort weight
auxdata.Qf=auxdata.coef*diag([wpos wvel]); 
auxdata.Qnoise=diag(ones(1,nw)); % noise covariance
auxdata.rescale=1 ; % to rescale the cost globally (only for numerical issues)

% Limb's properties with muscles
m=1.44;
auxdata.m=m;
l=0.35;
auxdata.l=l;
lg=0.21;
auxdata.lg=lg;
M=0.0588;
auxdata.M=M;
a1=2.5*1e-2;
auxdata.a1=a1;
a2=2.5*1e-2;
auxdata.a2=a2;
k=1621.6;
auxdata.k=k;
k0=810.8;
auxdata.k0=k0;
b=108.1;
auxdata.b=b;
b0=54.1; 
auxdata.b0=b0;
r1=-2.182*1e-2; % flexor
auxdata.r1=r1;
r2=2.182*1e-2; % extensor
auxdata.r2=r2;
l0lm1=-5.67*1e-2;
auxdata.l0lm1=l0lm1;
l0lm2=-0.436*1e-2;
auxdata.l0lm2=l0lm2;
g=0;% gravity (0 if horizontal movement)
auxdata.g=g;

% initial mean and covariance (augmented state = [m,c])
q0=25*pi/180; % initial position
qdot0=0; % initial vel
m0=[q0 qdot0 ]';
Pdiag0=diag([0 0]);
Ptriu0=triu(ones(ns),1);
Ptriu0(Ptriu0==1)=zeros(ns2-ns,1);
P0=Pdiag0+(Ptriu0+Ptriu0');
c0=P0(triu(true(size(P0)))); % initial covariance in vector form
auxdata.m0=m0;
auxdata.c0=c0;
  
% final state, time
mtarg=[65*pi/180 0]'; % final mean state (covariance free)
auxdata.mtarg=mtarg;
% Movement duration
T=.475; % seconds, or T=.400 seconds
tfguess=T;

% noise properties
% constant noise magnitude
sigma1=0;
% signal dependent noise magnitude
d1=4/a1*M;

auxdata.sigma1=sigma1;
auxdata.d1=d1;

%-------------------------------------------------------------------------%
%-----------  Find optimal solution (SOOC resolution)
%-------------------------------------------------------------------------%

run GenMain_forearmcocontraction

x=output.result.interpsolution.phase(1).state;
u=output.result.interpsolution.phase(1).control;
time=output.result.interpsolution.phase(1).time;
tf=time(end);

% Calculate costs
N=numel(time);
dt=tf/N;
effort=zeros(N-1,1);
for i=1:N-1  
    p = x(i,1:ns)'; % state (mean part)
    c = x(i,ns+1:end)'; % state (covariance part)
    u_=u(i,:)'; 
    Qt=auxdata.Q';
    Qel=auxdata.Q(triu(true(size(auxdata.Q))));
    effort(i)=(u_'*auxdata.R*u_);
end
EFFORT=sum(effort)*dt;

Qfel=auxdata.Qf(triu(true(size(auxdata.Qf))));
pf=x(end,1:ns)';
cf=x(end,ns+1:end)';
VARIANCE=Qfel'*cf;
TOTALCOST=EFFORT+VARIANCE;

% optimal trajectory
q = x(:,1);
qdot = x(:,2);

% optimal open-loop control
u1=u(:,1);
u2=u(:,2);
u1optpp = interpolate(time,u1,'linear','pp');  
u1opt = @(t) ppval(u1optpp,t);
u2optpp = interpolate(time,u2,'linear','pp');  
u2opt = @(t) ppval(u2optpp,t);

%-------------------------------------------------------------------------%
%------------- Simulate trajectories with noise
%-------------------------------------------------------------------------%
dt=0.001;
N=fix(tf/dt);
nrep=100;

for kk=1:nrep
    xrun(:,1)=x(1,1:ns)'; % initial state
    for i=1:N-1
        
      q_=xrun(1,i); qdot_=xrun(2,i); 
      ti=(i-1)*dt;
      Gnoise=[0 0 ; sigma1 d1*(abs(u1opt(ti)-u2opt(ti))^1.5+0.01*abs(u1opt(ti)+u2opt(ti))^1.5) ]/M*a1; % noise at thetaddot level
          
      % muscle 1
      l1dot=-a1*qdot_;
      tens1=(k0+k*u1opt(ti))*(r1*u1opt(ti)+l0lm1+a1*q_)-(b0+b*u1opt(ti))*l1dot;
      tau1=-a1*tens1;
      % muscle 2
      l2dot=-a2*qdot_;
      tens2=(k0+k*u2opt(ti))*(r2*u2opt(ti)+l0lm2+a2*q_)-(b0+b*u2opt(ti))*l2dot;
      tau2=-a2*tens2;  

      fdyn= [ qdot_ ; ...
             (tau1+tau2)/M ];
              
      xrun(:,i+1)=xrun(:,i)+fdyn*dt + Gnoise*sqrtm(auxdata.Qnoise)*randn(nw,1)*sqrt(dt);  
      
    end
  
    q_all(:,kk)=xrun(1,:)';
    qdot_all(:,kk)=xrun(2,:)'; 
    
  
end

time2=linspace(0,tf,N)';

for ii=1:size(q_all,1) 
   qstd(ii)=std(q_all(ii,:));
   qmean(ii)=mean(q_all(ii,:));
   qdotstd(ii)=std(qdot_all(ii,:));
   qdotmean(ii)=mean(qdot_all(ii,:));  
end

% cocontraction index
for i=1:numel(u1)
    minact(i)=min([u1(i), u2(i)]);
end
cocontID=mean(minact);

% other parameters of interest
[mval,idx]=max(abs(qdotmean));
maxvel=mval*180/pi;
asym=idx/length(qdotmean);
endpointstd=sqrt(x(end,ns+1))*180/pi;

%-------------------------------------------------------------------------%
%------------- Plot figures
%-------------------------------------------------------------------------%

figure('color','white');
width=6;
height=16;
set(gcf, 'Units', 'centimeters', 'OuterPosition', [0, 0, width, height]);
hold on;

subplot(311)
hold on;
plotshaded(time2',[qmean+qstd ; qmean ;qmean-qstd]*180/pi,'k'); hold on;
plot([tf+0.01 tf+0.01],[mtarg(1)*180/pi-2.5 mtarg(1)*180/pi+2.5],'-','color','k','linewidth',2)
for ii=1:10
    Xp=[time2' NaN];
    Yp=[q_all(:,ii)'*180/pi NaN];
    p = patch(Xp,Yp,'k');
    set(p,'edgecolor','k','linewidth',1,'edgealpha',0.2)
end
xlim([0 .5])
ylim([m0(1)*180/pi-5 mtarg(1)*180/pi+10])
xlabel('Time (s)')
ylabel('Position (deg)');
set(gca,'fontsize', 10);

subplot(312)
hold on;
plotshaded(time2',[qdotmean+qdotstd ; qdotmean ;qdotmean-qdotstd]*180/pi,'k');
for ii=1:10
    Xp=[time2' NaN];
    Yp=[qdot_all(:,ii)'*180/pi NaN];
    p = patch(Xp,Yp,'k');
    set(p,'edgecolor','k','linewidth',1,'edgealpha',0.1) 
end
xlabel('Time (s)')
xlim([0 .5])
ylabel('Velocity (deg/s)');
ylim([-15 250])
set(gca,'fontsize', 10);

subplot(313)
plot(time2',u1opt(time2),'color','k','linewidth',2); hold on;
plot(time2',u2opt(time2),'color',[.5 .5 .5],'linewidth',2);
xlabel('Time (s)')
xlim([0 .5])
ylabel('Muscle activations (a.u.)');
ylim([-0.05 2.25])
legend('Flexor u1','Extensor u2');
box off
set(gca,'fontsize', 10);

