% Figure 4. Two-link arm reaching experiment in a divergent force field.
% Figure 5. Endpoint stiffness geometry at the midpoint of movement path.

clear
clc
close all

%-------------------------------------------------------------------------%
%------------- Problem definition
%-------------------------------------------------------------------------%

ns=4; % dim of mean: [q1,q2,q1dot,q2dot]
ns2=ns*(ns+1)/2; % dim of covariance
nu=6; % dim of control: [u1,u2,u3,u4,u5,u6]
% Convention:
% shoulder flexor : 1
% shoulder extensor : 2
% elbow flexor : 3
% elbow extensor : 4
% double-joint flexor : 5
% double-joint extensor : 6
nw=2; % dim of noise
auxdata.ns=ns;
auxdata.ns2=ns2;
auxdata.nu=nu;

% cost parameters
auxdata.coef=1;
weightVariance=1e4; % variance weight
wpos=weightVariance;
auxdata.Q=auxdata.coef*diag([wpos,wpos,0,0]); 
auxdata.rescale=1; % to rescale the cost globally (only for numerical issues)
auxdata.R=diag(ones(1,nu)); % effort weight
auxdata.Qf=auxdata.Q;
Qf=auxdata.Qf;
auxdata.weight=1/2; % weight for Cartesian acceleration cost
auxdata.Qnoise=eye(nw); % noise covariance 

% endpoint force field
Fx=40; % divergent force field magnitude
auxdata.Fx=Fx;

% Segments' length
L1=0.3;
L2=0.35;

direc = 90*pi/180;
dist = 0.25;
    
% initial mean and covariance
cartini=[0 0.30];
xini0=cartini(1); yini0=cartini(2);
angini=Cart2Ang(cartini,L1,L2);
q10=angini(1); 
q20=angini(2);
m0=[q10 q20 0 0 ]'; 
Pdiag0=diag(zeros(1,ns)); % initial covariance
Ptriu0=triu(ones(ns),1);
Ptriu0(Ptriu0==1)=zeros(ns2-ns,1);
P0=Pdiag0+Ptriu0+Ptriu0';
c0=P0(triu(true(size(P0)))); % initial covariance in vector form

% final mean and covariance
cartend=[cos(direc); sin(direc)]*dist+cartini';
angend=Cart2Ang(cartend',L1,L2);
q1f=angend(1); 
q2f=angend(2);
mf=[q1f q2f 0 0 ]'; 
Pdiagf=diag(zeros(1,ns)); 
Ptriuf=triu(ones(ns),1);
Ptriuf(Ptriuf==1)=zeros(ns2-ns,1);
Pf=Pdiagf+Ptriuf+Ptriuf';
cf=Pf(triu(true(size(Pf))));

% initial mean and covariance 
auxdata.m0=m0;
auxdata.c0=c0;
% final mean and covariance (some components may be free)
auxdata.mf=mf;
auxdata.cf=cf;

% movement duration
T=0.75; % seconds
tfguess=T;

% noise properties 
% Magnitude of constant noise (acting in control space)
sigma1=0.025;
sigma2=sigma1;
auxdata.sigma1=sigma1;
auxdata.sigma2=sigma2;

%-------------------------------------------------------------------------%
%------------- Find optimal solution (SOOC resolution)
%-------------------------------------------------------------------------%

run GenMain_nonlinear_2_dof

x=output.result.interpsolution.phase(1).state;
u=output.result.interpsolution.phase(1).control;
time=output.result.interpsolution.phase(1).time;
tf=time(end);

% optimal state trajectories
q1opt = x(:,1);
q2opt = x(:,2);
q1dotopt = x(:,3);
q2dotopt = x(:,4); 
% optimal open-loop control
u1opt=u(:,1);
u2opt=u(:,2);
u3opt=u(:,3);
u4opt=u(:,4);
u5opt=u(:,5);
u6opt=u(:,6);
% elements of the covariance
q1optstd=sqrt(x(:,5));
q2optstd=sqrt(x(:,7));
q1dotoptstd=sqrt(x(:,10));
q2dotoptstd=sqrt(x(:,14));

% interpolate to create ui(t) functions
u1optpp = interpolate(time,u1opt,'linear','pp'); u1t = @(t) ppval(u1optpp,t);
u2optpp = interpolate(time,u2opt,'linear','pp'); u2t = @(t) ppval(u2optpp,t);
u3optpp = interpolate(time,u3opt,'linear','pp'); u3t = @(t) ppval(u3optpp,t);
u4optpp = interpolate(time,u4opt,'linear','pp'); u4t = @(t) ppval(u4optpp,t);
u5optpp = interpolate(time,u5opt,'linear','pp'); u5t = @(t) ppval(u5optpp,t);
u6optpp = interpolate(time,u6opt,'linear','pp'); u6t = @(t) ppval(u6optpp,t);

%-------------------------------------------------------------------------%
%------------- Simulate trajectories with noise
%-------------------------------------------------------------------------%

nrep=100; % number of samples
dt=0.001; % time step in seconds
N=fix(tf/dt);
q1all=zeros(N,nrep);
q2all=zeros(N,nrep);
q1dotall=zeros(N,nrep);
q2dotall=zeros(N,nrep);
vxall=zeros(N,nrep);
vyall=zeros(N,nrep);
vhandall=zeros(N,nrep);
for k=1:nrep
    xrun(:,1)=x(1,1:ns)'; % initial state found
    for i=1:N-1
        
        ti=(i-1)*dt;
        q1=xrun(1,i); q2=xrun(2,i); q1dot=xrun(3,i); q2dot=xrun(4,i);   
        u1=u1t(ti); u2=u2t(ti); u3=u3t(ti); u4=u4t(ti); u5=u5t(ti); u6=u6t(ti);   

        q1ddot=(250000*((q1dot*((1081*u1)/10 + 541/10))/625 + (q1dot*((1081*u2)/10 + 541/10))/625 - (567*q2dot^2*sin(q2))/6250 + (7*((8108*u5)/5 + 4054/5)*((7*q1)/250 + (7*q2)/200 - (2749*u5)/50000 - 7147/50000))/250 + (7*((8108*u6)/5 + 4054/5)*((7*q1)/250 + (7*q2)/200 + (2749*u6)/50000 + 1343/100000))/250 + (7*((7*q1dot)/250 + (7*q2dot)/200)*((1081*u5)/10 + 541/10))/250 + (7*((7*q1dot)/250 + (7*q2dot)/200)*((1081*u6)/10 + 541/10))/250 - (((8108*u1)/5 + 4054/5)*((3491*u1)/100000 - q1/25 + 2269/25000))/25 + (((8108*u2)/5 + 4054/5)*(q1/25 + (3491*u2)/100000 + 2793/100000))/25 - (567*q1dot*q2dot*sin(q2))/3125))/(9*(3888*cos(q2)^2 - 4925)) - (50000*(54*cos(q2) + 35)*((q2dot*((1081*u3)/10 + 541/10))/1600 + (q2dot*((1081*u4)/10 + 541/10))/1600 + (567*q1dot^2*sin(q2))/6250 + (7*((8108*u5)/5 + 4054/5)*((7*q1)/250 + (7*q2)/200 - (2749*u5)/50000 - 7147/50000))/200 + (7*((8108*u6)/5 + 4054/5)*((7*q1)/250 + (7*q2)/200 + (2749*u6)/50000 + 1343/100000))/200 + (7*((7*q1dot)/250 + (7*q2dot)/200)*((1081*u5)/10 + 541/10))/200 + (7*((7*q1dot)/250 + (7*q2dot)/200)*((1081*u6)/10 + 541/10))/200 - (((8108*u3)/5 + 4054/5)*((1091*u3)/50000 - q2/40 + 709/12500))/40 + (((8108*u4)/5 + 4054/5)*(q2/40 + (1091*u4)/50000 - 109/25000))/40))/(63*(3888*cos(q2)^2 - 4925)) + Fx*((7*cos(q1 + q2))/20 + (3*cos(q1))/10)*((250000*((7*sin(q1 + q2))/20 + (3*sin(q1))/10))/(9*(3888*cos(q2)^2 - 4925)) - (2500*sin(q1 + q2)*(54*cos(q2) + 35))/(9*(3888*cos(q2)^2 - 4925)));
        q2ddot=Fx*((625*sin(q1 + q2)*(3024*cos(q2) + 3935))/(63*(3888*cos(q2)^2 - 4925)) - (50000*(54*cos(q2) + 35)*((7*sin(q1 + q2))/20 + (3*sin(q1))/10))/(63*(3888*cos(q2)^2 - 4925)))*((7*cos(q1 + q2))/20 + (3*cos(q1))/10) - (50000*(54*cos(q2) + 35)*((q1dot*((1081*u1)/10 + 541/10))/625 + (q1dot*((1081*u2)/10 + 541/10))/625 - (567*q2dot^2*sin(q2))/6250 + (7*((8108*u5)/5 + 4054/5)*((7*q1)/250 + (7*q2)/200 - (2749*u5)/50000 - 7147/50000))/250 + (7*((8108*u6)/5 + 4054/5)*((7*q1)/250 + (7*q2)/200 + (2749*u6)/50000 + 1343/100000))/250 + (7*((7*q1dot)/250 + (7*q2dot)/200)*((1081*u5)/10 + 541/10))/250 + (7*((7*q1dot)/250 + (7*q2dot)/200)*((1081*u6)/10 + 541/10))/250 - (((8108*u1)/5 + 4054/5)*((3491*u1)/100000 - q1/25 + 2269/25000))/25 + (((8108*u2)/5 + 4054/5)*(q1/25 + (3491*u2)/100000 + 2793/100000))/25 - (567*q1dot*q2dot*sin(q2))/3125))/(63*(3888*cos(q2)^2 - 4925)) + (12500*(3024*cos(q2) + 3935)*((q2dot*((1081*u3)/10 + 541/10))/1600 + (q2dot*((1081*u4)/10 + 541/10))/1600 + (567*q1dot^2*sin(q2))/6250 + (7*((8108*u5)/5 + 4054/5)*((7*q1)/250 + (7*q2)/200 - (2749*u5)/50000 - 7147/50000))/200 + (7*((8108*u6)/5 + 4054/5)*((7*q1)/250 + (7*q2)/200 + (2749*u6)/50000 + 1343/100000))/200 + (7*((7*q1dot)/250 + (7*q2dot)/200)*((1081*u5)/10 + 541/10))/200 + (7*((7*q1dot)/250 + (7*q2dot)/200)*((1081*u6)/10 + 541/10))/200 - (((8108*u3)/5 + 4054/5)*((1091*u3)/50000 - q2/40 + 709/12500))/40 + (((8108*u4)/5 + 4054/5)*(q2/40 + (1091*u4)/50000 - 109/25000))/40))/(441*(3888*cos(q2)^2 - 4925));

        fdyn= [  q1dot; ...
                 q2dot; ...
                 q1ddot; ...
                 q2ddot ];   

        Gnoise =[  0, 0 ; ...
                   0, 0 ; ...
                  -(250000*sigma1)/(9*(3888*cos(q2)^2 - 4925)),       (50000*sigma2*(54*cos(q2) + 35))/(63*(3888*cos(q2)^2 - 4925)) ; ...
                  (50000*sigma1*(54*cos(q2) + 35))/(63*(3888*cos(q2)^2 - 4925)), -(12500*sigma2*(3024*cos(q2) + 3935))/(441*(3888*cos(q2)^2 - 4925))];

        xrun(:,i+1)=xrun(:,i)+fdyn*dt + ...
                    Gnoise*sqrtm(auxdata.Qnoise)*randn(nw,1)*sqrt(dt);      
    end
     
    q1all(:,k)=xrun(1,:)';
    q2all(:,k)=xrun(2,:)';   
    q1dotall(:,k)=xrun(3,:)'; 
    q2dotall(:,k)=xrun(4,:)'; 
    
    vxall(:,k)=-(L1*sin(q1all(:,k))+L2*sin(q1all(:,k)+q2all(:,k))).*q1dotall(:,k) ...
               - L2*sin(q1all(:,k)+q2all(:,k)).*q2dotall(:,k);
    vyall(:,k)= (L1*cos(q1all(:,k))+L2*cos(q1all(:,k)+q2all(:,k))).*q1dotall(:,k) ...
               + L2*cos(q1all(:,k)+q2all(:,k)).*q2dotall(:,k);

    vhandall(:,k)=sqrt(vxall(:,k).^2+vyall(:,k).^2);
end
time2=linspace(0,tf,N)';

q1std=zeros(1,N);
q1mean=zeros(1,N);
q2std=zeros(1,N);
q2mean=zeros(1,N);
q1dotstd=zeros(1,N);
q1dotmean=zeros(1,N);
q2dotstd=zeros(1,N);
q2dotmean=zeros(1,N);
vhandmean=zeros(1,N);
vhandstd=zeros(1,N);
for ii=1:size(q1all,1) 
   % positions
   q1std(ii)=std(q1all(ii,:));
   q1mean(ii)=mean(q1all(ii,:));
   q2std(ii)=std(q2all(ii,:));
   q2mean(ii)=mean(q2all(ii,:));
   % velocities
   q1dotstd(ii)=std(q1dotall(ii,:));
   q1dotmean(ii)=mean(q1dotall(ii,:));
   q2dotstd(ii)=std(q2dotall(ii,:));
   q2dotmean(ii)=mean(q2dotall(ii,:));
   % Cartesian vel
   vhandmean(ii)=mean(vhandall(ii,:));
   vhandstd(ii)=std(vhandall(ii,:));
end

% Empirical final mean
xfmeanempirical=[q1mean(end) q2mean(end) q1dotmean(end) q2dotmean(end)];
% Empirical final covariance
Pfempirical=cov([q1all(end,:)' q2all(end,:)' q1dotall(end,:)' q2dotall(end,:)']);

% x axis and y axis velocities
vx=-(L1*sin(q1opt)+L2*sin(q1opt+q2opt)).*q1dotopt - L2*sin(q1opt+q2opt).*q2dotopt;
vy= (L1*cos(q1opt)+L2*cos(q1opt+q2opt)).*q1dotopt + L2*cos(q1opt+q2opt).*q2dotopt;

posxopt =L1*cos(q1opt)+L2*cos(q1opt+q2opt);
posyopt =L1*sin(q1opt)+L2*sin(q1opt+q2opt);

%-------------------------------------------------------------------------%
%------------- Plot figures
%-------------------------------------------------------------------------%

figure('color','white');
width=10;
height=12;
set(gcf, 'Units', 'centimeters', 'OuterPosition', [10, 10, width, height]);
hold on;
xtarg=2.5e-2/2*cos(linspace(0,2*pi,100))+cartend(1);
ytarg=2.5e-2/2*sin(linspace(0,2*pi,100))+cartend(2);
plot(xtarg,ytarg,'-b','linewidth',2)
hold on;
xall=zeros(N,nrep); yall=zeros(N,nrep);
posxendall=zeros(1,nrep); posyendall=zeros(1,nrep);
for ii=1:nrep
    posx =L1*cos(q1all(:,ii))+L2*cos(q1all(:,ii)+q2all(:,ii));
    posy =L1*sin(q1all(:,ii))+L2*sin(q1all(:,ii)+q2all(:,ii)); 
    xall(:,ii)=posx;
    yall(:,ii)=posy;
    posxendall(ii) =posx(end);
    posyendall(ii) =posy(end);     
    if ii<=20 % plot only a few samples!
        Xp=[posx' NaN];
        Yp=[posy' NaN];
        p = patch(Xp,Yp,'k');
        set(p,'edgecolor','k','linewidth',1,'edgealpha',0.2)
        plot(posx(end),posy(end),'.k');
    end
end
plot(mean(xall,2),mean(yall,2),'k','linewidth',3);
plot(posxopt,posyopt,'-r','linewidth',1);
% Confidence ellipse from sampled trajectories
[xellip2,yellip2]=confidenceEllipse(posxendall',posyendall');
plot(xellip2,yellip2,'-g','linewidth',2);
plot([-.03 -.03],[0.25 0.6],':k');
plot([.03 .03],[0.25 0.6],':k');
xlabel('x (m)')
ylabel('y (m)')
axis equal;
xlim([-0.2 0.2])
ylim([0.25 .60])
set(gca,'fontsize', 10);


% Finger Cartesian velocity
figure('color','white');
width=8;
height=11;
set(gcf, 'Units', 'centimeters', 'OuterPosition', [10, 10, width, height]);
hold on;
plotshaded(time2',[vhandmean+vhandstd ; vhandmean ;vhandmean-vhandstd],'k');
vopt=sqrt(vx.^2+vy.^2);
plot(time2',vhandmean,'linewidth',3,'color','k')
plot(time,vopt,'-r','linewidth',1);
xlim([0 tf])
ylabel('Velocity (m/s)');
xlabel('Time (s)')
set(gca,'fontsize', 10);

figure('color','white');
width=12;
height=12;
set(gcf, 'Units', 'centimeters', 'OuterPosition', [10, 10, width, height]);
subplot(221)
hold on;
plotshaded(time',[(q1opt+q1optstd)' ; q1opt' ;(q1opt-q1optstd)']*180/pi,'r');
plotshaded(time2',[q1mean+q1std ; q1mean ;q1mean-q1std]*180/pi,'k');
xlim([0 tf])
xlabel('Time (s)')
ylabel('q1 (deg)');
set(gca,'fontsize', 10);


subplot(222)
hold on;
plotshaded(time',[(q1dotopt+q1dotoptstd)' ; q1dotopt' ;(q1dotopt-q1dotoptstd)']*180/pi,'r');
plotshaded(time2',[q1dotmean+q1dotstd ; q1dotmean ;q1dotmean-q1dotstd]*180/pi,'k');
xlim([0 tf])
xlabel('Time (s)')
ylabel('q1dot (deg/s)');
set(gca,'fontsize', 10);

subplot(223)
hold on;
plotshaded(time',[(q2opt+q2optstd)' ; q2opt' ;(q2opt-q2optstd)']*180/pi,'r');
plotshaded(time2',[q2mean+q2std ; q2mean ;q2mean-q2std]*180/pi,'k');
xlim([0 tf])
xlabel('Time (s)')
ylabel('q2 (deg)');
set(gca,'fontsize', 10);

subplot(224)
hold on;
plotshaded(time',[(q2dotopt+q2dotoptstd)' ; q2dotopt' ;(q2dotopt-q2dotoptstd)']*180/pi,'r');
plotshaded(time2',[q2dotmean+q2dotstd ; q2dotmean ;q2dotmean-q2dotstd]*180/pi,'k');
xlim([0 tf])
xlabel('Time (s)')
ylabel('q2dot (deg/s)');
set(gca,'fontsize', 10);

% Plot motor commands, muscle tensions and stiffnesses
figure('color','white');
width=20;
height=12;
set(gcf, 'Units', 'centimeters', 'OuterPosition', [10, 10, width, height]);
subplot(132)
hold on;
colors=[0 102 0 ; 0 51 204 ; 153 0 0]/256;
for i=1:6
    if i==1 || i==3 || i==5
        plot(time,u(:,i),'linewidth',2,'color',colors((i+1-rem(i+1,2))/2,:)); 
    else
        plot(time,u(:,i),'linewidth',2,'linestyle','-.','color',colors((i+1-rem(i+1,2))/2,:)); 
    end
end
ylim([0 1.1])
xlim([0 tf])
xlabel('Time (s)');
ylabel('Controls (u_i)')
[lgd,legendIcons]=legend('Sh-Flex.','Sh-Ext.','El-Flex.','El-Ext.','Dbl-Flex.','Dbl-Ext.');

set(gca,'fontsize', 10);

% taken from associated symbolic function
A=[0.04 0;0.04 0;0 0.025;0 0.025;0.028 0.035;0.028 0.035];
Kjoint = zeros(2,2,numel(u1opt));
Kendpoint = zeros(2,2,numel(u1opt));
Bjoint=cell(1,numel(u1opt));
for tt=1:numel(u1opt)
    
    Ku= [(8108*u1opt(tt))/5 + 4054/5 ; ...
         (8108*u2opt(tt))/5 + 4054/5 ; ...
         (8108*u3opt(tt))/5 + 4054/5 ; ...
         (8108*u4opt(tt))/5 + 4054/5 ; ...
         (8108*u5opt(tt))/5 + 4054/5 ; ...
         (8108*u6opt(tt))/5 + 4054/5 ];
    Bu= [(1081*u1opt(tt))/10 + 541/10; ...
         (1081*u2opt(tt))/10 + 541/10; ...
         (1081*u3opt(tt))/10 + 541/10; ...
         (1081*u4opt(tt))/10 + 541/10; ...
         (1081*u5opt(tt))/10 + 541/10 ; ...
         (1081*u6opt(tt))/10 + 541/10];
    Jtt=[-L1*sin(q1opt(tt))-L2*sin(q1opt(tt)+q2opt(tt)), -L2*sin(q1opt(tt)+q2opt(tt)) ; ...
           L1*cos(q1opt(tt))+L2*cos(q1opt(tt)+q2opt(tt)),  L2*cos(q1opt(tt)+q2opt(tt)) ];
   
    Kjoint(:,:,tt)=A'*diag(Ku)*A;

    HessianMatrix1=[-L1*cos(q1opt(tt))-L2*cos(q1opt(tt)+q2opt(tt)) -L2*cos(q1opt(tt)+q2opt(tt)) ; ...
                   -L2*cos(q1opt(tt)+q2opt(tt)) -L2*cos(q1opt(tt)+q2opt(tt)) ] ;
    HessianMatrix2=[-L1*sin(q1opt(tt))-L2*sin(q1opt(tt)+q2opt(tt)) -L2*sin(q1opt(tt)+q2opt(tt)) ; ...
                   -L2*sin(q1opt(tt)+q2opt(tt)) -L2*sin(q1opt(tt)+q2opt(tt)) ] ;   
               
    Kendpoint(:,:,tt)=inv(Jtt)'*(squeeze(Kjoint(:,:,tt)))*inv(Jtt);
    Bjoint{tt}=A'*diag(Bu)*A;
    
end

subplot(131)
hold on;

for i=1:size(Kendpoint,1)
    for j=i:size(Kendpoint,2)
        if i==1 && j==1
            plot(time,squeeze(Kendpoint(i,j,:)),'linewidth',2,'color',[0 0 0]); 
        elseif i==1 && j==2
        	plot(time,squeeze(Kendpoint(i,j,:)),'linewidth',2,'color',[0 0 0]+0.7);    
        else
        	plot(time,squeeze(Kendpoint(i,j,:)),'linewidth',2,'color',[0 0 0]+0.4);   
        end
    end
end
ylim([-30 150])
xlim([0 tf])
[lgd2,legendIcons2]=legend('Sxx','Sxy','Syy');
xlabel('Time (s)');
ylabel('Endpoint stiffness (N/m)')
set(gca,'fontsize', 10);

subplot(133)
muscleTensions =[ (q1dotopt.*((1081.*u1opt)/10 + 541/10))/25 - ((8108.*u1opt)/5 + 4054/5).*((3491.*u1opt)/100000 - q1opt/25 + 2269/25000) , ...
                  (q1dotopt.*((1081.*u2opt)/10 + 541/10))/25 + ((8108.*u2opt)/5 + 4054/5).*(q1opt/25 + (3491.*u2opt)/100000 + 2793/100000), ...
                  (q2dotopt.*((1081.*u3opt)/10 + 541/10))/40 - ((8108.*u3opt)/5 + 4054/5).*((1091.*u3opt)/50000 - q2opt/40 + 709/12500), ...
                  (q2dotopt.*((1081.*u4opt)/10 + 541/10))/40 + ((8108.*u4opt)/5 + 4054/5).*(q2opt/40 + (1091.*u4opt)/50000 - 109/25000), ...
                  ((8108.*u5opt)/5 + 4054/5).*((7.*q1opt)/250 + (7.*q2opt)/200 - (2749.*u5opt)/50000 - 7147/50000) + ((7.*q1dotopt)/250 + (7.*q2dotopt)/200).*((1081.*u5opt)/10 + 541/10), ...
                  ((8108.*u6opt)/5 + 4054/5).*((7.*q1opt)/250 + (7.*q2opt)/200 + (2749.*u6opt)/50000 + 1343/100000) + ((7.*q1dotopt)/250 + (7.*q2dotopt)/200).*((1081.*u6opt)/10 + 541/10)];
 
hold on;
for i=1:6
    if i==1 || i==3 || i==5
        plot(time,abs(muscleTensions(:,i)),'linewidth',2,'color',colors((i+1-rem(i+1,2))/2,:)); 
    else
        plot(time,abs(muscleTensions(:,i)),'linewidth',2,'linestyle','-.','color',colors((i+1-rem(i+1,2))/2,:)); 
    end
    
end
ylim([0 300])
xlim([0 tf])
xlabel('Time (s)');
ylabel('Muscle tensions (N)')
[lgd3,legendIcons3]=legend('Sh-Flex.','Sh-Ext.','El-Flex.','El-Ext.','Dbl-Flex.','Dbl-Ext.');
set(gca,'fontsize', 10);

% Endpoint stiffness in N/m
Kendpointmean=squeeze(mean(Kendpoint,3));

% Endpoint stiffness at midpath
figure('color','white');
width=12;
height=12;
set(gcf, 'Units', 'centimeters', 'OuterPosition', [10, 10, width, height]);
tmids=find(posyopt>=(cartini(2)+(cartend(2)-cartini(2))/2));
tmid=tmids(1);
[xellip3,yellip3]=stiff2ell(squeeze(Kendpoint(:,:,tmid)),[posxopt(tmid) posyopt(tmid)]');
plot(xellip3,yellip3,'r','linewidth',2)
hold on;
% plot a reference stiffness ellipse of 10 N/m
[xellip3,yellip3]=stiff2ell([10 0 ; 0 10],[posxopt(tmid) posyopt(tmid)]');
plot(xellip3,yellip3,'b','linewidth',1); 
axis equal;
axis off;

